@@ -1,7 +1,7 @@
 /**********************************************************************
  *  mainwindow.cpp
  **********************************************************************
- * Copyright (C) 2019 MX Authors
+ * Copyright (C) 2017 MX Authors
  *
  * Authors: Dolphin Oracle
  *          MX Linux <http://mxlinux.org>
@@ -23,7 +23,7 @@
  **********************************************************************/
 
 
-#include "about.h"
+
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
 #include "version.h"
@@ -34,14 +34,13 @@
 
 #include <QDebug>
 
-MainWindow::MainWindow(const QStringList& args)  :
+MainWindow::MainWindow(const QStringList& args) :
     ui(new Ui::MainWindow)
 {
-    qDebug().noquote() << QCoreApplication::applicationName() << "version:" << VERSION;
     ui->setupUi(this);
     setWindowFlags(Qt::Window); // for the close, min and max buttons
     setup();
-    ui->comboBoxUsbList->addItems(buildUsbList());
+    ui->combo_Usb->addItems(buildUsbList());
     this->adjustSize();
 }
 
@@ -53,33 +52,20 @@ MainWindow::~MainWindow()
 
 void MainWindow::makeUsb(const QString &options)
 {
-    //get a device value if action is on a partition
-    QString device_to_check = device;
-    if (device.contains("nvme")){
-        device_to_check = device.section("p",0,0);
-    }
-    if (device.contains("mmc")){
-        device_to_check = device.section("p",0,0);
-    }
-
-    if (device.contains("sd")){
-        device_to_check = device.left(3);
-    }
 
     // check amount of io on device before copy, this is in sectors
-    start_io = cmd->getCmdOut("cat /sys/block/" + device_to_check + "/stat |awk '{print $7}'").toInt();
+    start_io = cmd->getOutput("cat /sys/block/" + device + "/stat |awk '{print $7}'", QStringList() << "quiet").toInt();
     ui->progressBar->setMinimum(start_io);
     qDebug() << "start io is " << start_io;
     ui->progressBar->setMaximum(iso_sectors+start_io);
     qDebug() << "max progress bar is " << ui->progressBar->maximum();
     //clear partitions
-    //qDebug() << cmd->getCmdOut("live-usb-maker gui partition-clear --color=off -t " + device);
+    qDebug() << cmd->getOutput("live-usb-maker gui partition-clear --color=off -t " + device);
     QString cmdstr = options;
-    //QString cmdstr = "echo test options";
     setConnections();
-    qDebug() << cmd->getCmdOut(cmdstr);
+    qDebug() << cmd->getOutput(cmdstr, QStringList() << "slowtick");
     //label drive
-    //labeldrive();
+    labeldrive();
 }
 
 // setup versious items first time program runs
@@ -98,63 +84,66 @@ void MainWindow::setup()
     height = this->heightMM();
     ui->lineEditFSlabel->setText("USB-DATA");
 
+    QRegExp rx("\\w*");
+    QValidator *validator = new QRegExpValidator(rx, this);
 }
 
 // Build the option list to be passed to live-usb-maker
 QString MainWindow::buildOptionList()
 {
-    device = ui->comboBoxUsbList->currentText().split(" ").at(0);
+    device = ui->combo_Usb->currentText().split(" ").at(0);
     label = ui->lineEditFSlabel->text();
-    QString partoption;
-    QString options;
-
     QString format = ui->comboBoxDataFormat->currentText();
     if (format.contains("fat32")){
         format = "vfat";
     }
+    qDebug() << "usb device" << device << "label " << label;
+    QString options;
+    options = QString("live-usb-maker gui --format=" + format + " --color=off -t " + device);
 
-
-    if (ui->comboBoxPartitionTableType->isEnabled()) {
-        //0=defaults, 1=msdos, 2=gpt
-        switch(ui->comboBoxPartitionTableType->currentIndex()){
-        case 0:
-            partoption = "defaults";
-            break;
-        case 1:
-            partoption = "msdos";
-            break;
-        case 2:
-            partoption = "gpt";
-            break;
-        default: partoption = "defaults";
-        }
-    } else {
-        partoption = "part";
-    }
-
-
-    qDebug() << "partition is" << device << "label " << label;
-    options = QString("su-to-root -X -c '/usr/lib/formatusb/formatusb_lib \"" + device + "\" " + format + " \"" + label + "\" " + partoption + "'");
     qDebug() << "Options: " << options;
     return options;
 }
 
+// label new drive
+void MainWindow::labeldrive()
+{
+    QString cmdstr;
+    QString target;
+    QString partnum = "1";
+    if ( device.contains("mmc")) {
+        partnum = "p1";
+    }
+
+    if ( ui->comboBoxDataFormat->currentText() == "fat32") {
+        cmdstr = QString("fatlabel /dev/" + device + partnum + " \"%1\"").arg(label);
+    }
+
+    if ( ui->comboBoxDataFormat->currentText() == "ext4") {
+        cmdstr = QString("e2label /dev/" + device + partnum + " \"%1\"").arg(label);
+    }
+
+    if ( ui->comboBoxDataFormat->currentText() == "ntfs") {
+        cmdstr = QString("ntfslabel /dev/" + device + partnum + " \"%1\"").arg(label);
+    }
+
+    if ( ui->comboBoxDataFormat->currentText() == "exfat") {
+        cmdstr = QString("exfatlabel /dev/" + device + partnum + " \"%1\"").arg(label);
+    }
+    qDebug() << "label string" << cmdstr;
+    cmd->getOutput(cmdstr);
+}
+
 // cleanup environment when window is closed
 void MainWindow::cleanup()
 {
-    QString log_name = "/tmp/formatusb.log";
-    system("[ -f " + log_name.toUtf8() + " ] && rm " + log_name.toUtf8());
+
 }
 
 // build the USB list
 QStringList MainWindow::buildUsbList()
-{   QString drives;
-    if (ui->checkBoxshowpartitions->isChecked()){
-        drives = cmd->getCmdOut("lsblk -nlo NAME,SIZE,LABEL,TYPE -I 3,8,22,179,259 |grep -v disk");
-    } else {
-        drives = cmd->getCmdOut("lsblk --nodeps -nlo NAME,SIZE,MODEL,VENDOR -I 3,8,22,179,259 ");
-    }
-
+{
+    QString drives = cmd->getOutput("lsblk --nodeps -nlo name,size,model,vendor -I 3,8,22,179,259");
     return removeUnsuitable(drives.split("\n"));
 }
 
@@ -163,22 +152,14 @@ QStringList MainWindow::removeUnsuitable(const QStringList &devices)
 {
     QStringList list;
     QString name;
-    bool showall = ui->checkBoxShowAll->isChecked();
     for (const QString &line : devices) {
         name = line.split(" ").at(0);
-        if (!showall){
-            if (system(cli_utils.toUtf8() + "is_usb_or_removable " + name.toUtf8()) == 0) {
-                if (cmd->getCmdOut(cli_utils + "get_drive $(get_live_dev) ") != name) {
-                    list << line;
-                }
-            }
-        } else {
-            if (cmd->getCmdOut(cli_utils + "get_drive $(get_live_dev) ") != name) {
+        if (system(cli_utils.toUtf8() + "is_usb_or_removable " + name.toUtf8()) == 0) {
+            if (cmd->getOutput(cli_utils + "get_drive $(get_live_dev) ") != name) {
                 list << line;
             }
         }
     }
-
     return list;
 }
 
@@ -194,53 +175,47 @@ void MainWindow::cmdDone()
     ui->progressBar->setValue(ui->progressBar->maximum());
     setCursor(QCursor(Qt::ArrowCursor));
     ui->buttonBack->setEnabled(true);
-    if (cmd->exitCode() == 0 && cmd->exitStatus() == QProcess::NormalExit) {
+    if (cmd->getExitCode() == 0) {
         QMessageBox::information(this, tr("Success"), tr("Format successful!"));
     } else {
         QMessageBox::critical(this, tr("Failure"), tr("Error encountered in the Format process"));
     }
     cmd->disconnect();
-    timer.stop();
 }
 
 // set proc and timer connections
 void MainWindow::setConnections()
 {
-    timer.start(1000);
-    connect(cmd, &QProcess::readyRead, this, &MainWindow::updateOutput);
-    connect(cmd, &QProcess::started, this, &MainWindow::cmdStart);
-    connect(&timer, &QTimer::timeout, this, &MainWindow::updateBar);
-    connect(cmd, static_cast<void (QProcess::*)(int)>(&QProcess::finished), this, &MainWindow::cmdDone);
+    connect(cmd, &Cmd::outputAvailable, this, &MainWindow::updateOutput);
+    connect(cmd, &Cmd::started, this, &MainWindow::cmdStart);
+    connect(cmd, &Cmd::runTime, this, &MainWindow::updateBar);
+    connect(cmd, &Cmd::finished, this, &MainWindow::cmdDone);
 
 }
 
 void MainWindow::updateBar()
 {
-    if (!ui->checkBoxshowpartitions->isChecked()){
-        int current_io = cmdprog->getCmdOut("cat /sys/block/" + device + "/stat | awk '{print $7}'").toInt();
-        ui->progressBar->setValue(current_io);
-    }
+    int current_io = cmdprog->getOutput("cat /sys/block/" + device + "/stat | awk '{print $7}'", QStringList() << "quiet" << "slowtick").toInt();
+    ui->progressBar->setValue(current_io);
 }
 
-void MainWindow::updateOutput()
+void MainWindow::updateOutput(QString out)
 {
     // remove escape sequences that are not handled by code
-    QString out = cmd->readAll();
     out.remove("[0m").remove("]0;").remove("").remove("").remove("[1000D").remove("[74C|").remove("[?25l").remove("[?25h").remove("[0;36m").remove("[1;37m");
-//    if (out.contains("[10D[K")) { // escape sequence used to display the progress percentage
-//        out.remove("[10D[K");
-//        ui->outputBox->moveCursor(QTextCursor::StartOfLine);
-//        QKeyEvent *event = new QKeyEvent(QEvent::KeyPress, Qt::Key_K, Qt::ControlModifier);
-//        QCoreApplication::postEvent(ui->outputBox, event);
-//        QString out_prog = out;
-//        ui->progressBar->setValue(out_prog.remove(" ").remove("%").toInt());
-//    }
+    if (out.contains("[10D[K")) { // escape sequence used to display the progress percentage
+        out.remove("[10D[K");
+        ui->outputBox->moveCursor(QTextCursor::StartOfLine);
+        QKeyEvent *event = new QKeyEvent(QEvent::KeyPress, Qt::Key_K, Qt::ControlModifier);
+        QCoreApplication::postEvent(ui->outputBox, event);
+        QString out_prog = out;
+        ui->progressBar->setValue(out_prog.remove(" ").remove("%").toInt());
+    }
 
     ui->outputBox->insertPlainText(out);
 
     QScrollBar *sb = ui->outputBox->verticalScrollBar();
     sb->setValue(sb->maximum());
-    qApp->processEvents();
 }
 
 // Next button clicked
@@ -249,20 +224,21 @@ void MainWindow::on_buttonNext_clicked()
 
     // on first page
     if (ui->stackedWidget->currentIndex() == 0) {
-        if (ui->comboBoxUsbList->currentText() == "") {
+        if (ui->combo_Usb->currentText() == "") {
             QMessageBox::critical(this, tr("Error"), tr("Please select a USB device to write to"));
             return;
         }
 
         //confirm action
         int ans;
-        QString msg = tr("These actions will destroy all data on \n\n") + ui->comboBoxUsbList->currentText().simplified() + "\n\n " + tr("Do you wish to continue?");
-        ans = QMessageBox::warning(this, windowTitle(), msg,
-                                   QMessageBox::Yes, QMessageBox::No);
-        if (ans != QMessageBox::Yes) {
-            return;
-        }
-        if (cmd->state() != QProcess::NotRunning) {
+        QString msg = tr("These actions will destroy all data on \n\n") + ui->combo_Usb->currentText().simplified() + "\n\n " + tr("Do you wish to continue?");
+                    ans = QMessageBox::warning(this, windowTitle(), msg,
+                                               QMessageBox::Yes, QMessageBox::No);
+                    if (ans != QMessageBox::Yes) {
+                        return;
+                    }
+
+        if (cmd->isRunning()) {
             ui->stackedWidget->setCurrentWidget(ui->outputPage);
             return;
         }
@@ -296,41 +272,67 @@ void MainWindow::on_buttonBack_clicked()
 void MainWindow::on_buttonAbout_clicked()
 {
     this->hide();
-    displayAboutMsgBox(tr("About %1").arg(this->windowTitle()), "<p align=\"center\"><b><h2>" + this->windowTitle() +"</h2></b></p><p align=\"center\">" +
+    QMessageBox msgBox(QMessageBox::NoIcon,
+                       tr("About") + " Format USB", "<p align=\"center\"><b><h2>Format USB</h2></b></p><p align=\"center\">" +
                        tr("Version: ") + VERSION + "</p><p align=\"center\"><h3>" +
                        tr("Program for formatting USB devices") +
                        "</h3></p><p align=\"center\"><a href=\"http://mxlinux.org\">http://mxlinux.org</a><br /></p><p align=\"center\">" +
-                       tr("Copyright (c) MX Linux") + "<br /><br /></p>",
-                       "/usr/share/doc/formatusb/license.html", tr("%1 License").arg(this->windowTitle()), true);
+                       tr("Copyright (c) MX Linux") + "<br /><br /></p>");
+    QPushButton *btnLicense = msgBox.addButton(tr("License"), QMessageBox::HelpRole);
+    QPushButton *btnChangelog = msgBox.addButton(tr("Changelog"), QMessageBox::HelpRole);
+    QPushButton *btnCancel = msgBox.addButton(tr("Cancel"), QMessageBox::NoRole);
+    btnCancel->setIcon(QIcon::fromTheme("window-close"));
+
+    msgBox.exec();
+
+    Cmd cmd;
+    if (msgBox.clickedButton() == btnLicense) {
+
+        QString user = cmd.getOutput("logname");
+        if (system("command -v mx-viewer") == 0) { // use mx-viewer if available
+            system("su " + user.toUtf8() + " -c \"mx-viewer file:///usr/share/doc/formatusb/license.html 'Format USB " + tr("License").toUtf8() + "'\"&");
+        } else {
+            system("su " + user.toUtf8() + " -c \"xdg-open file:///usr/share/doc/formatusb/license.html\"&");
+        }
+    } else if (msgBox.clickedButton() == btnChangelog) {
+    QDialog *changelog = new QDialog(this);
+    changelog->resize(600, 500);
+
+    QTextEdit *text = new QTextEdit;
+    text->setReadOnly(true);
+    text->setText(cmd.getOutput("zless /usr/share/doc/" + QFileInfo(QCoreApplication::applicationFilePath()).fileName()  + "/changelog.gz"));
+
+    QPushButton *btnClose = new QPushButton(tr("&Close"));
+    btnClose->setIcon(QIcon::fromTheme("window-close"));
+    connect(btnClose, &QPushButton::clicked, changelog, &QDialog::close);
+
+    QVBoxLayout *layout = new QVBoxLayout;
+    layout->addWidget(text);
+    layout->addWidget(btnClose);
+    changelog->setLayout(layout);
+    changelog->exec();
+}
     this->show();
 }
 
 // Help button clicked
 void MainWindow::on_buttonHelp_clicked()
 {
-    QString url = "file:///usr/share/doc/formatusb/help/formatusb.html";
-    displayDoc(url, tr("%1 Help").arg(this->windowTitle()), true);
+    QString url = "https://mxlinux.org/wiki/help-files/help-format-usb";
+    QString exec = "xdg-open";
+    if (system("command -v mx-viewer") == 0) { // use mx-viewer if available
+        exec = "mx-viewer";
+    }
+
+    Cmd c;
+    QString user = c.getOutput("logname");
+    QString cmd = QString("su " + user + " -c \"" + exec + " " + url + "\"&");
+    system(cmd.toUtf8());
 }
 
 void MainWindow::on_buttonRefresh_clicked()
 {
-    ui->comboBoxUsbList->clear();
-    ui->comboBoxUsbList->addItems(buildUsbList());
+    ui->combo_Usb->clear();
+    ui->combo_Usb->addItems(buildUsbList());
 }
 
-
-void MainWindow::on_checkBoxShowAll_clicked()
-{
-    on_buttonRefresh_clicked();
-}
-
-void MainWindow::on_checkBoxshowpartitions_clicked()
-{
-     on_buttonRefresh_clicked();
-     if ( ui->checkBoxshowpartitions->isChecked()){
-         ui->comboBoxPartitionTableType->setEnabled(false);
-     }
-     if ( ! ui->checkBoxshowpartitions->isChecked()){
-         ui->comboBoxPartitionTableType->setEnabled(true);
-     }
-}
